import re
from time import time

from django.conf import settings
from django.http import HttpRequest, HttpResponse

from django_logbox.threading import ServerLogInsertThread
from django_logbox.utils import _get_client_ip, _get_server_ip, get_log_data

logbox_logger_thread = ServerLogInsertThread.get_instance()


class LogboxMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request: HttpRequest):
        timestamp = time()
        response = self.get_response(request)

        if not self._filter_requests(request) or not self._filter_responses(response):
            return response

        if not hasattr(request, "logbox_logged"):
            logbox_logger_thread.put_serverlog(
                get_log_data(
                    timestamp=timestamp,
                    request=request,
                    response=response,
                    exception=None,
                ),
            )
            request.logbox_logged = True

        return response

    def process_exception(self, request: HttpRequest, exception: Exception):
        data = get_log_data(time(), request, None, exception)

        logbox_logger_thread.put_serverlog(data)
        request.logbox_logged = True

        return None

    @staticmethod
    def _filter_requests(request: HttpRequest) -> bool:
        return (
            LogboxMiddleware._filter_client_ip(request)
            and LogboxMiddleware._filter_server_ip(request)
            and LogboxMiddleware._filter_path(request)
        )

    @staticmethod
    def _filter_client_ip(request: HttpRequest) -> bool:
        """
        Filter requests based on client IP.

        :return: True if the request should be logged, False otherwise.
        """
        return (
            _get_client_ip(request)
            not in settings.LOGBOX_SETTINGS["LOGGING_CLIENT_IPS_TO_EXCLUDE"]
        )

    @staticmethod
    def _filter_server_ip(request: HttpRequest):
        """
        Filter requests based on server IP.

        :return: True if the request should be logged, False otherwise.
        """
        return (
            _get_server_ip(request)
            not in settings.LOGBOX_SETTINGS["LOGGING_SERVER_IPS_TO_EXCLUDE"]
        )

    @staticmethod
    def _filter_path(request: HttpRequest) -> bool:
        """Filter requests based on path patterns."""

        return not any(
            re.match(path, request.path)
            for path in settings.LOGBOX_SETTINGS["LOGGING_PATHS_TO_EXCLUDE"]
        )

    @staticmethod
    def _filter_responses(response: HttpResponse):
        return response.status_code in settings.LOGBOX_SETTINGS["LOGGING_STATUS_CODES"]



# Generated by Django 5.1.1 on 2024-10-27 07:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_logbox", "0003_rename_exception_serverlog_exception_type_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="serverlog",
            name="client_ip",
            field=models.GenericIPAddressField(
                help_text="IP address of the client making the request.",
                verbose_name="client_ip",
            ),
        ),
        migrations.AlterField(
            model_name="serverlog",
            name="exception_message",
            field=models.TextField(
                help_text="Detailed message provided by the exception.",
                null=True,
                verbose_name="exception_message",
            ),
        ),
        migrations.AlterField(
            model_name="serverlog",
            name="exception_type",
            field=models.CharField(
                help_text="Class name or type of the exception, if any occurred.",
                max_length=255,
                null=True,
                verbose_name="exception_type",
            ),
        ),
        migrations.AlterField(
            model_name="serverlog",
            name="method",
            field=models.CharField(
                help_text="HTTP method used for the request, e.g., 'GET', 'POST', 'PUT'.",
                max_length=10,
                verbose_name="method",
            ),
        ),
        migrations.AlterField(
            model_name="serverlog",
            name="path",
            field=models.CharField(
                help_text="The endpoint path requested, excluding the domain, e.g., '/api/v1/users/'.",
                max_length=255,
                verbose_name="path",
            ),
        ),
        migrations.AlterField(
            model_name="serverlog",
            name="querystring",
            field=models.TextField(
                help_text="Query parameters of the request as a URL-encoded string, e.g., 'param1=value1&param2=value2'.",
                null=True,
                verbose_name="querystring",
            ),
        ),
        migrations.AlterField(
            model_name="serverlog",
            name="request_body",
            field=models.TextField(
                help_text="Body content of the request, usually JSON or form data. Null if no body was sent.",
                null=True,
                verbose_name="request_body",
            ),
        ),
        migrations.AlterField(
            model_name="serverlog",
            name="server_ip",
            field=models.GenericIPAddressField(
                help_text="IP address of the server handling the request.",
                verbose_name="server_ip",
            ),
        ),
        migrations.AlterField(
            model_name="serverlog",
            name="status_code",
            field=models.IntegerField(
                help_text="HTTP status code of the response, e.g., 200, 404, 500.",
                verbose_name="status_code",
            ),
        ),
        migrations.AlterField(
            model_name="serverlog",
            name="timestamp",
            field=models.DateTimeField(
                help_text="Date and time when this log entry was created.",
                verbose_name="timestamp",
            ),
        ),
        migrations.AlterField(
            model_name="serverlog",
            name="traceback",
            field=models.TextField(
                help_text="Full traceback of the exception for debugging purposes.",
                null=True,
                verbose_name="traceback",
            ),
        ),
        migrations.AlterField(
            model_name="serverlog",
            name="user_agent",
            field=models.TextField(
                help_text="User agent string from the client's request header, providing browser and OS details.",
                max_length=255,
                null=True,
                verbose_name="user_agent",
            ),
        ),
    ]



from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from ua_parser import parse

from .models import ServerLog


@admin.register(ServerLog)
class ServerLogAdmin(admin.ModelAdmin):
    list_display = (
        "method",
        "path",
        "status_code",
        "user_agent_summary",
        "timestamp",
        "exception_type",
        "server_ip",
        "client_ip",
    )

    readonly_fields = (
        "method",
        "path",
        "status_code",
        "user_agent",
        "querystring",
        "request_body",
        "timestamp",
        "exception_type",
        "exception_message",
        "traceback",
        "server_ip",
        "client_ip",
    )

    fieldsets = (
        (
            _("Request Information"),
            {
                "fields": (
                    "timestamp",
                    "method",
                    "path",
                    "status_code",
                    "user_agent_details",
                    "querystring",
                    "request_body",
                ),
            },
        ),
        (
            _("Exception Details"),
            {
                "fields": (
                    "exception_type",
                    "exception_message",
                    "traceback",
                ),
            },
        ),
        (
            _("IP Addresses"),
            {
                "fields": (
                    "server_ip",
                    "client_ip",
                ),
            },
        ),
    )
    list_display_links = ("method", "path")
    search_fields = ("status_code", "exception_message")
    list_filter = ("method", "status_code", "path", "timestamp")

    @staticmethod
    @admin.display(description=_("User-Agent Summary"))
    def user_agent_summary(obj):
        if not obj.user_agent:
            return None
        _parsed_useragent_result = parse(obj.user_agent)

        device_summary = (
            _parsed_useragent_result.device.family
            if _parsed_useragent_result.device
            else "X"
        )
        os_summary = (
            _parsed_useragent_result.os.family
            if _parsed_useragent_result.device
            else "X"
        )
        user_agent_summary = (
            _parsed_useragent_result.user_agent if _parsed_useragent_result.os else "X"
        )

        return f"{device_summary}, {os_summary}, {user_agent_summary}"

    @staticmethod
    @admin.display(description=_("User-Agent Details"))
    def user_agent_details(obj):
        if not obj.user_agent:
            return None
        _parsed_useragent_result = parse(obj.user_agent)

        device_details = (
            f"Device: {_parsed_useragent_result.device.family}({_parsed_useragent_result.device.brand}, {_parsed_useragent_result.device.model})"
            if _parsed_useragent_result.device
            else _("No device data found.")
        )
        os_details = (
            f"OS: {_parsed_useragent_result.os.family}({_parsed_useragent_result.os.major}.{_parsed_useragent_result.os.minor}.{_parsed_useragent_result.os.patch})"
            if _parsed_useragent_result.os
            else _("No OS data fount.")
        )
        user_agent_details = (
            f"User-Agent: {_parsed_useragent_result.user_agent.family}({_parsed_useragent_result.os.major}.{_parsed_useragent_result.os.minor}.{_parsed_useragent_result.os.patch})"
            if _parsed_useragent_result.os
            else _("No User-Agent data found.")
        )

        return format_html(
            f"<p>{obj.user_agent}</p>"
            f"<li>{device_details}</li>"
            f"<li>{os_details}</li>"
            f"<li>{user_agent_details}</li>"
        )

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False



from django.db.models import (
    CharField,
    DateTimeField,
    GenericIPAddressField,
    IntegerField,
    Model,
    TextField,
)
from django.utils.translation import gettext_lazy as _


class ServerLog(Model):
    # http
    method = CharField(
        _("method"),
        help_text=_("HTTP method used for the request, e.g., 'GET', 'POST', 'PUT'."),
        max_length=10,
    )
    path = CharField(
        _("path"),
        help_text=_(
            "The endpoint path requested, excluding the domain, e.g., '/api/v1/users/'."
        ),
        max_length=255,
    )
    status_code = IntegerField(
        _("status_code"),
        help_text=_("HTTP status code of the response, e.g., 200, 404, 500."),
    )
    user_agent = TextField(
        _("user_agent"),
        help_text=_(
            "User agent string from the client's request header, providing browser and OS details."
        ),
        max_length=255,
        null=True,
    )
    querystring = TextField(
        _("querystring"),
        help_text=_(
            "Query parameters of the request as a URL-encoded string, e.g., 'param1=value1&param2=value2'."
        ),
        null=True,
    )
    request_body = TextField(
        _("request_body"),
        help_text=_(
            "Body content of the request, usually JSON or form data. Null if no body was sent."
        ),
        null=True,
    )

    # log
    timestamp = DateTimeField(
        _("timestamp"),
        help_text=_("Date and time when this log entry was created."),
    )
    exception_type = CharField(
        _("exception_type"),
        help_text=_("Class name or type of the exception, if any occurred."),
        max_length=255,
        null=True,
    )
    exception_message = TextField(
        _("exception_message"),
        help_text=_("Detailed message provided by the exception."),
        null=True,
    )
    traceback = TextField(
        _("traceback"),
        help_text=_("Full traceback of the exception for debugging purposes."),
        null=True,
    )

    # ip
    server_ip = GenericIPAddressField(
        _("server_ip"), help_text=_("IP address of the server handling the request.")
    )
    client_ip = GenericIPAddressField(
        _("client_ip"), help_text=_("IP address of the client making the request.")
    )

    def __str__(self) -> str:
        return f"{self.timestamp} {self.method} {self.path} {self.status_code}"

    class Meta:
        verbose_name = _("Server Log")
        verbose_name_plural = _("Server Logs")
        ordering = ("-timestamp",)



# Generated by Django 5.1.1 on 2024-09-26 14:11

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ServerLog",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("method", models.CharField(max_length=10, verbose_name="method")),
                ("path", models.CharField(max_length=255, verbose_name="path")),
                ("status_code", models.IntegerField(verbose_name="status_code")),
                (
                    "user_agent",
                    models.CharField(
                        max_length=255, null=True, verbose_name="user_agent"
                    ),
                ),
                (
                    "querystring",
                    models.TextField(null=True, verbose_name="querystring"),
                ),
                (
                    "request_body",
                    models.TextField(null=True, verbose_name="request_body"),
                ),
                ("timestamp", models.DateTimeField(verbose_name="timestamp")),
                (
                    "duration",
                    models.DecimalField(
                        decimal_places=6, max_digits=10, verbose_name="duration"
                    ),
                ),
                ("exception", models.TextField(null=True, verbose_name="exception")),
                ("traceback", models.TextField(null=True, verbose_name="traceback")),
                ("server_ip", models.GenericIPAddressField(verbose_name="server_ip")),
                ("client_ip", models.GenericIPAddressField(verbose_name="client_ip")),
            ],
        ),
    ]
